clear all
close all
clc

load yankees.txt

yyear = yankees(:,1);
yPCT = yankees(:,2); %winning percentage in year t
yOBP = yankees(:,5); %team on-base percentage in year t
ySLG = yankees(:,6); %team slugging average in year t
yERA = yankees(:,8); %team earned run average in year t

load redsox.txt

ryear = redsox(:,1);
rPCT =  redsox(:,2); %winning percentage in year t
rOBP =  redsox(:,5); %team on-base percentage in year t
rSLG =  redsox(:,6); %team slugging average in year t
rERA =  redsox(:,8); %team earned run average in year t

tic

%% SUR model

N = length(yyear);
M = 2;


yX = [ones(size(yyear)) yOBP ySLG yERA];
rX = [ones(size(ryear)) rOBP rSLG rERA];

yy = yPCT;
ry = rPCT;

k1 = size(yX,2);
k2 = size(rX,2);
k = k1+k2;

y = zeros(M*N,1);
X = zeros(M*N,k);
for ii=1:N
X(M*ii-1:M*ii,:) = [yX(ii,:) zeros(1,k2)
                  zeros(1,k1) rX(ii,:)];
y(M*ii-1:M*ii,1) = [yy(ii,1)
                  ry(ii,1)];
end




%% Apriorni hyperparametry a nastaveni Gibbsova vzorkovace

S = 15000;
S0 = 5000;


b0 = zeros(k,1);
V0 = 4*eye(k);
nu0 = 0;
invH0 = zeros(M);
H0 = eye(M);

b_Gibbs = zeros(k,S);
H_Gibbs = zeros(M,M,S);

%% Gibbsuv vzorkovac
fprintf('Posteriorni simulace\n');
fprintf('0%%       50%%      100%% \n');
fprintf('|');
%prvni beh
sumV1 = zeros(k,k);
sumb1 = zeros(k,1);
for ii=1:N
   sumV1 = sumV1+(X(M*ii-1:M*ii,:)')*H0*X(M*ii-1:M*ii,:); %cast sumy v Koop 6.50
   sumb1 = sumb1+(X(M*ii-1:M*ii,:)')*H0*y(M*ii-1:M*ii,1); %cast sumy v Koop 6.51)
end

V1 = inv(V0)+sumV1; %neinvertovane V1 ... Koop 6.50 pro ucely matalbu pak nasledne deleni zleva (presnejsi vypocet inverze
b1 = V1\(V0\b0+sumb1); %inverze pomoci deleni zleva "\" ... Koop 6.51

%podminena hustota pro b
b_Gibbs(:,1) = b1+norm_rnd(inv(V1)); %Koop 6.49

%podminena hustota pro H
sumH = zeros(M,M);
for ii=1:N
   sumH = sumH+ (y(M*ii-1:M*ii,1)-X(M*ii-1:M*ii,:)*b_Gibbs(:,1))*(y(M*ii-1:M*ii,1)-X(M*ii-1:M*ii,:)*b_Gibbs(:,1))'; %cast sumy v Koop 6.54
end
nu1 = N+nu0; %Koop 6.52
H1 = inv(invH0+sumH); %Koop 6.54

%podminena hustota pro H
H_Gibbs(:,:,1) = wish_rnd(H1,nu1); %Koop 6.49



%dalsi behy

for s=2:S
sumV1 = zeros(k,k);
sumb1 = zeros(k,1);
for ii=1:N
   sumV1 = sumV1+(X(M*ii-1:M*ii,:)')*H_Gibbs(:,:,s-1)*X(M*ii-1:M*ii,:); %cast sumy v Koop 6.50
   sumb1 = sumb1+(X(M*ii-1:M*ii,:)')*H_Gibbs(:,:,s-1)*y(M*ii-1:M*ii,1); %cast sumy v Koop 6.51)
end

V1 = inv(V0)+sumV1; %neinvertovane V1 ... Koop 6.50 pro ucely matalbu pak nasledne deleni zleva (presnejsi vypocet inverze
b1 = V1\(V0\b0+sumb1); %inverze pomoci deleni zleva "\" ... Koop 6.51

%podminena hustota pro b
b_Gibbs(:,s) = b1+norm_rnd(inv(V1)); %Koop 6.49

%podminena hustota pro H
sumH = zeros(M,M);
for ii=1:N
   sumH = sumH+ (y(M*ii-1:M*ii,1)-X(M*ii-1:M*ii,:)*b_Gibbs(:,s))*(y(M*ii-1:M*ii,1)-X(M*ii-1:M*ii,:)*b_Gibbs(:,s))'; %cast sumy v Koop 6.54
end
nu1 = N+nu0; %Koop 6.52
H1 = inv(invH0+sumH); %Koop 6.54

%podminena hustota pro H
H_Gibbs(:,:,s) = wish_rnd(H1,nu1); %Koop 6.49


    if mod(s/S*100,5) == 0;    
      fprintf('|');
    end
end

%% vyhozeni prvnich S0 vzorku
b_S1 = b_Gibbs(:,S0+1:S);
H_S1 = H_Gibbs(:,:,S0+1:S);
r_S1 = H_S1(2,1,:)./(sqrt(H_S1(1,1,:).*H_S1(2,2,:))); %korelace nahodne slozky

b_mean = mean(b_S1,2); %posteriorni stredni hodnota 
H_mean = mean(H_S1,3);
r_mean = mean(r_S1);
b_var = mean(b_S1.^2,2)-b_mean.^2; %posteriorni rozptyl
H_var = mean(H_S1.^2,3)-H_mean.^2;
r_var = mean(r_S1.^2)-r_mean.^2;



%% HPDI pro beta a r (korelace mezi nahodnymi slozkami jednotlivych modelu)

hperc=0.95; %HPDI s vyuzitim funkce "quantile" (dostupna ve sdtatistickem i ekonometrickem toolboxu)
HPDIb_l = quantile(b_S1',0.025);
HPDIb_u = quantile(b_S1',0.975);
HPDIr_l = quantile(r_S1,0.025);
HPDIr_u = quantile(r_S1,0.975);



fprintf('\n');
fprintf('Hotovo!\n\n');

fprintf('Apriorni a posteriorni hustoty pro parametry beta a korelace nahodnych slozek\n');
fprintf('                  Prior       Posterior      %3.1f%% HPDI \n',hperc*100);
fprintf('   ------------------------------------\n');


fprintf('Rovnice pro Yankees\n');
fprintf('   ------------------------------------\n');
for i=1:k1
fprintf('Beta%1.0f          %10.4f   %10.4f   [%10.4f, %10.4f]\n',[i b_mean(i) sqrt(b_var(i)) HPDIb_l(i) HPDIb_u(i)]);
end
fprintf('   ------------------------------------\n');
fprintf('Rovnice pro Red Sox\n');
fprintf('   ------------------------------------\n');
for i=k1+1:k1+k2
fprintf('Beta%1.0f          %10.4f   %10.4f   [%10.4f, %10.4f]\n',[i b_mean(i) sqrt(b_var(i)) HPDIb_l(i) HPDIb_u(i)]);
end
fprintf('   ------------------------------------\n');
fprintf('Korelace nahodnych slozek mezi rovnicemi\n');
fprintf('   ------------------------------------\n');
fprintf('corr(e1,e2)    %10.4f   %10.4f   [%10.4f, %10.4f]\n',[r_mean sqrt(r_var) HPDIr_l HPDIr_u]);
fprintf('   ------------------------------------\n');